/**
 * @class DSS.component.RecurrenceSelection
 * @extends DSS.component.Base
 * Recurrence Selection Panel
 */
Ext.define('DSS.component.RecurrenceSelection', {
    alias: ['widget.dssRecurrenceSelection'],
    extend: 'DSS.component.Base',
    config: {
        /**
         * {Boolean} Display daytime selections, allowing the selection of daytime based
         * events like sunrise and sunset +/- a delay.
         */
        enableDaytime: true
    },

    constructor: function(config) {
        this.initConfig(config);
        this.callParent(arguments);
    },

    /** @private */
    recurring: true,
    /** @private */
    weekdaySelection: null,
    /** @private */
    dateField: null,
    /** @private */
    isAbsolute: true,
    /** @private */
    timeField: null,
    /** @private */
    prepostField: null,

    initComponent: function() {
        var me = this,
            timeContainerItems;

        me.weekdaySelection = Ext.create('DSS.component.WeekdaySelection');
        me.dateField = Ext.create('Ext.form.field.Date', {
            allowBlank: false,
            hidden: true,
            name: 'onetimeDate',
            format: 'd.m.Y',
            value: new Date()
        });

        me.prepostField = Ext.create('Ext.button.Cycle', {
            margin: '0 0 0 20',
            hidden: true,
            itemId: 'cyc-prepost',
            menu: {
                items: [{
                    checked: true,
                    itemId: 'chk-pre',
                    text: _('before')
                }, {
                    itemId: 'chk-post',
                    text: _('after')
                }]
            },
            showText: true,
            width: 120
        });

        me.timeField = Ext.create('Ext.container.Container', {
            defaultType: 'numberfield',
            defaults: {
                allowBlank: false,
                allowDecimals: false,
                width: 50
            },
            items: [{
                itemId: 'hours',
                value: 12,
                minValue: 0,
                maxValue: 23
            }, {
                itemId: 'minutes',
                value: 0,
                minValue: 0,
                maxValue: 59
            }, {
                itemId: 'seconds',
                value: 0,
                minValue: 0,
                maxValue: 59
            }, {
                xtype: 'label',
                itemId: 'lblHMS',
                text: '(' + _('hh:mm:ss') + ')',
                align: 'middle',
                margin: '5 0 0 10'
            },
            me.prepostField
            ],
            layout: 'column',
            width: 350
        });

        if (this.enableDaytime) {
            timeContainerItems = [
                {
                    xtype: 'cycle',
                    itemId: 'cyc-daytime',
                    listeners: {
                        change: function(cycle, item, options) {
                             me.setAbsoluteTime(item.itemId == 'chk-time');
                        }
                    },
                    menu: {
                        items: [{
                            checked: true,
                            itemId: 'chk-time',
                            text: _('Time of day')
                        }, {
                            itemId: 'chk-sunrise',
                            text: _('Sunrise')
                        }, {
                            itemId: 'chk-sunset',
                            text: _('Sunset')
                        }]
                    },
                    margin: '0 20 0 0',
                    showText: true,
                    width: 180
                },
                me.timeField
            ];

        } else {
            // not using daytime
            timeContainerItems = [me.timeField];
        }

        Ext.apply(me, {
            //layout: 'anchor',
            items: [{
                    xtype: 'fieldcontainer',
                    itemId: 'time',
                    fieldLabel: _('Time'),
                    layout: 'column',
                    items: timeContainerItems
                }, {
                    itemId: 'days',
                    xtype: 'fieldcontainer',
                    fieldLabel: _('Recurrence'),
                    layout: 'column',
                    items: [
                        {
                            xtype: 'cycle',
                            itemId: 'cyc-recurrence',
                            listeners: {
                                change: function(cycle, item, options) {
                                     me.setRecurring(item.itemId == 'chk-weekly');
                                }
                            },
                            menu: {
                                items: [{
                                    checked: true,
                                    itemId: 'chk-weekly',
                                    text: _('Weekly event')
                                }, {
                                    text: _('Specific dates')
                                }]
                            },
                            showText: true,
                            width: 180
                        }, {
                            xtype: 'container',
                            columnWidth: 0.5,
                            itemId: 'recurrenceSelection',
                            items: [
                                me.weekdaySelection,
                                me.dateField
                            ],
                            padding: '0 0 0 20'
                        }
                    ]
                }
            ]
        });
        me.callParent(arguments);
    },

    /**
     * Switch to change between displaying of recurring or one time event
     * @param recurring {Boolean} true: set recurring event, false: one time event
     */
    setRecurring: function(recurring) {
        var me = this;
        if (me.recurring == recurring) {
            return;
        }

        if (recurring) {
            me.dateField.hide();
            me.weekdaySelection.show();
        } else {
            me.weekdaySelection.hide();
            me.dateField.show();
        }
        me.recurring = recurring;
    },

    /**
     * Switch to change between displaying of absolute or relatively timed event
     * @param isAbsolute {Boolean} true: set absolute time, false: one time event
     */
    setAbsoluteTime: function(isAbsolute) {
        var me = this,
            sec;
        if (me.isAbsolute == isAbsolute)  {
            return;
        }
        sec = me.timeField.getComponent('seconds');
        if (isAbsolute) {
            me.prepostField.hide();
            me.timeField.getComponent('hours').setValue(12);
            me.timeField.getComponent('minutes').setValue(0);
            me.timeField.getComponent('lblHMS').setText('(' + _('hh:mm:ss') + ')');
            sec.setValue(0);
            sec.show();
        } else {
            me.prepostField.show();
            me.timeField.getComponent('hours').setValue(0);
            me.timeField.getComponent('minutes').setValue(0);
            me.timeField.getComponent('lblHMS').setText('(' + _('hh:mm') + ')');
            // hide seconds field on delay
            sec.hide();
        }
        me.isAbsolute = isAbsolute;
    },

    /**
     * Reset all values to default (unset) or specified values
     * @param values {Object} (optional) values to reset to - default if unset
     */
    resetComponent: function(values) {
        var me = this,
            timebox = me.timeField,
            cycRec = me.getComponent('days').getComponent('cyc-recurrence'),
            cycDay = me.getComponent('time').getComponent('cyc-daytime');

        if (values === undefined) {
            me.setRecurring(true);
            timebox.getComponent('hours').setValue(12);
            timebox.getComponent('minutes').setValue(0);
            timebox.getComponent('seconds').setValue(0);
            cycRec.setActiveItem(cycRec.menu.items.first());
            me.weekdaySelection.resetComponent();
            me.dateField.setValue(new Date());
            if (this.enableDaytime) {
                cycDay.setActiveItem(cycDay.menu.items.first());
                me.setAbsoluteTime(true);
            }
        } else {
            me.setRecurring(!!values.recurring);
            cycRec.setActiveItem(!!values.recurring ? cycRec.menu.items.first() : cycRec.menu.items.last());
            if (this.enableDaytime) {
                if (values.daytime === null) {
                    cycDay.setActiveItem('chk-time');
                } else {
                    if (values.daytime == 'sunrise+') {
                        cycDay.setActiveItem('chk-sunrise');
                        me.prepostField.setActiveItem('chk-post');
                    } else if (values.daytime == 'sunrise-') {
                        cycDay.setActiveItem('chk-sunrise');
                        me.prepostField.setActiveItem('chk-pre');
                    } else if (values.daytime == 'sunset+') {
                        cycDay.setActiveItem('chk-sunset');
                        me.prepostField.setActiveItem('chk-post');
                    } else if (values.daytime == 'sunset-') {
                        cycDay.setActiveItem('chk-sunset');
                        me.prepostField.setActiveItem('chk-pre');
                    }
                }
            }
            timebox.getComponent('hours').setValue(values.hours);
            timebox.getComponent('minutes').setValue(values.minutes);
            timebox.getComponent('seconds').setValue(values.seconds);
            if (values.recurring) {
                me.weekdaySelection.resetComponent(values.recurrence);
                me.dateField.setValue(new Date());
            } else {
                me.weekdaySelection.resetComponent();
                // TODO: add support for multiple dates
                me.dateField.setValue(values.recurrence[0]);
            }
        }
    },

    /**
     * Returns whether this form's values are in a valid state
     * @return {Boolean} True if valid, false otherwise
     */
    isValid: function() {
        var timebox = this.timeField;
        if (this.recurring) {
            this.weekdaySelection.validate();
        }
        return (
            (
                (this.recurring && this.weekdaySelection.isValid())
                || (!this.recurring && this.dateField.isValid())
            )
            && timebox.getComponent('hours').isValid()
            && timebox.getComponent('minutes').isValid()
            && timebox.getComponent('seconds').isValid()
        );
    },

    /**
     * Gets the string of the currently selected daytime
     * @return {String} A string matching (sunrise|sunset)[+-] or null if absolute time is used
     */
    getDayTime: function() {
        var id = this.getComponent('time').getComponent('cyc-daytime').getActiveItem().itemId,
            prepost,
            ret;
        if (id === 'chk-time') {
            return null;
        } else {
            if (id === 'chk-sunrise') {
                ret = 'sunrise';
            } else if (id === 'chk-sunset') {
                ret = 'sunset';
            }

            prepost = this.prepostField.getActiveItem().itemId;
            if (prepost === 'chk-post') {
                ret += '+';
            } else if (prepost === 'chk-pre') {
                ret += '-';
            }
            return ret;
        }
    },

    /**
     * Returns the currently set form values
     * Use isValid() to determine if current values are valid
     * @return {Object} Object of form:
     *      recurring: {Boolean} True if recurrence is filled with weekdays (array of ints)
     *                  or false for dates (array of Ext.Date)
     *      recurrence: {Array} Array of Ext.Date objects or array of ints with
     *                  selected day index (Sun=0..Sat=6)
     *      daytime: {String} currently selected daytime combination: null or sunrise+ or sunset-
     *      hours: {Number} Hour component of time (delay if daytime is set)
     *      minutes: {Number} Minute component of time (delay if daytime is set)
     *      seconds: {Number} Second component of time (delay if daytime is set)
     */
    getValues: function() {
        var timebox = this.timeField,
            ret = {
                recurring: this.recurring,
                recurrence: this.recurring
                            ? this.weekdaySelection.getValues()
                            : [this.dateField.getValue()], // TODO: support multiple dates
                daytime: this.getDayTime(),
                hours: timebox.getComponent('hours').getValue(),
                minutes: timebox.getComponent('minutes').getValue(),
                seconds: timebox.getComponent('seconds').getValue()
            };
        return ret;
    }
});
